--[[
Snowflake - a library to provide a dropdown menu interface for professions

Copyright (C) 2006-2008  Prat Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local AceGUI = LibStub("AceGUI-3.0")

--[[---------------------------------------------------------------------------------
Button
------------------------------------------------------------------------------------]]
local Type = 'SnowflakeTitle'
local Version = 1

local function OnAcquire( self)
end

local function OnRelease( self)
end

local function Control_OnEnter( btn)
	btn.obj:Fire( 'OnEnter')
end

local function Control_OnLeave( btn)
	btn.obj:Fire( 'OnLeave')
end

local function Control_OnMouseDown( btn)
	btn.obj:Fire( 'OnMouseDown')
	AceGUI:ClearFocus()
end

local function Control_OnMouseUp( btn)
	btn.obj:Fire( 'OnMouseUp')
end

local function SetSelect( self, value)
end

local function InitImage( image, info)
	image:SetDesaturated( info.disabled)
	if info.checked then
		if info.icon then
			image:SetTexture( info.icon)
			image:SetWidth( info.iconWidth or 16)
			image:SetHeight( info.iconHeight or 16)
		else
			image:SetTexture( 'Interface\\Buttons\\UI-CheckBox-Check')
			image:SetWidth( 24)
			image:SetHeight( 24)
		end
		image:SetVertexColor( 1, 1, 1, 1)
		image:Show()
	elseif info.icon then
		image:SetTexture( info.icon)
		image:SetWidth( info.iconWidth or 16)
		image:SetHeight( info.iconHeight or 16)
		image:SetVertexColor( 1, 1, 1, 1)
		image:Show()
	else
		image:Hide()
	end
	if info.iconLeft and info.iconRight and info.iconTop and info.iconBottom then
		image:SetTexCoord( info.iconLeft, info.iconRight, info.iconTop, info.iconBottom)
	elseif info.icon and info.icon:find( '^Interface\\Icons\\') then
		image:SetTexCoord( 0.05, 0.95, 0.05, 0.95)
	else
		image:SetTexCoord( 0, 1, 0, 1)
	end
end

local function InitLabel( label, info)
	local height = info.textHeight or UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT
	label:SetFontObject( GameFontNormalSmall)
	label:SetFont( STANDARD_TEXT_FONT, height)
	label:SetJustifyH( info.justifyH or 'LEFT')
	label:SetText( info.text or '')
	return height
end

local function SetInfo( self, info)
	self.userdata.token    = info.token or info.text or self.frame:GetName()
	self.userdata.disabled = info.isPlain or info.disabled
	self.userdata.maxWidth = info.maxWidth or 999999
	InitImage( self.image, info)
	local height = InitLabel( self.label, info)
	self.frame:SetHeight( height + 6)
end

local function GetMinWidth( self)
	local minWidth = 28 + self.label:GetStringWidth() + 20
	return math.min( minWidth, self.userdata.maxWidth)
end

local function Constructor()
	local id  = AceGUI:GetNextWidgetNum( Type)
	local self = { num = id, type = Type }
	
	local frame = CreateFrame( 'Button', Type .. id, UIParent)
	frame.obj = self
	frame:SetHeight( 16)
	frame:SetAlpha( 1)
	frame:EnableMouse( true)
	frame:SetFrameStrata( 'FULLSCREEN_DIALOG')
	frame:SetScript( 'OnEnter', Control_OnEnter)
	frame:SetScript( 'OnLeave', Control_OnLeave)
	frame:SetScript( 'OnMouseDown', Control_OnMouseDown)
	frame:SetScript( 'OnMouseUp',   Control_OnMouseUp)
	self.frame = frame
	
	local image = frame:CreateTexture( nil, 'ARTWORK')
	image:SetTexture( 'Interface\\Buttons\\UI-CheckBox-Check')
	image:SetPoint( 'CENTER', frame, 'LEFT', 12, 0)
	image:SetWidth( 24)
	image:SetHeight( 24)
	image:Hide()
	self.image = image
	
	local label = frame:CreateFontString( nil, 'ARTWORK')
	label:SetFontObject( GameFontHighlightSmall)
	label:SetFont( STANDARD_TEXT_FONT, UIDROPDOWNMENU_DEFAULT_TEXT_HEIGHT)
	label:SetPoint( 'LEFT', image, 'LEFT', 4, 0)
	label:SetPoint( 'RIGHT', frame, 'LEFT', -4, 0)
	self.label = label
	
	self.OnAcquire = OnAcquire
	self.OnRelease = OnRelease
	
	self.SetInfo     = SetInfo
	self.SetSelect   = SetSelect
	self.GetMinWidth = GetMinWidth
	
	AceGUI:RegisterAsWidget( self)
	return self
end

AceGUI:RegisterWidgetType( Type, Constructor, Version)
